package nrs.jcards;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Definition of a counter
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class Counter
{
	private String name = null;
	private double value = 0.0;
	private double increment = 1.0;
	
	/** Creates a new instance of Counter */
	public Counter(String name, double value, double increment)
	{
		this.name = name;
		this.value = value;
		this.increment = increment;
	}

	/** Create a new instance of Counter */
	public Counter(String name, String value, String increment) throws NumberFormatException
	{
		this.name = name;
		this.value = Double.valueOf(value).doubleValue();
		this.increment = Double.valueOf(increment).doubleValue();
	}
	
	/**
	 * Set the name of the counter
	 * @param newName name of the counter
	 */
	public void setName(String newName)
	{
		name = newName;
	}
	
	/** Get the name of the counter */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Set the value of the counter
	 * @param newValue value between Double.MIN_VALUE and Double.MAX_VALUE
	 */
	public void setValue(double newValue)
	{
		value = newValue;
	}
	
	/** Get the value */
	public double getValue()
	{
		return value;
	}
	
	/**
	 * Set the increment
	 * @param newIncrement increment value which can be plus or minus but not zero
	 * If a zero is passed in it is ignored and the increment remains unchanged
	 */
	public void setIncrement(double newIncrement)
	{
		if (newIncrement != 0) {
			increment = newIncrement;
		}
	}
	
	/** Get the increment */
	public double getIncrement()
	{	
		return increment;
	}
	
	/** Increment the counter */
	public void increment()
	{
		value += increment;
	}
	
	/** Rollback the counter */
	public void rollback()
	{
		value -= increment;
	}
	
	/** Return a String representation of this Counter */
	public String toString()
	{
		String s = name +", "+value +", ";
		if (increment > 0) s += "+";
		s += increment;
		
		return s;
	}
}
