package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import nrs.jcards.JCards;

/**
 * Show version information and some system info
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class AboutDialog extends Dialog implements ActionListener
{
	private TextArea info = null;

	/**
	 * Creates a new instance of AboutDialog
	 * @param parentFrame the owning frame
	 */
	public AboutDialog(Frame parentFrame)
	{
		super(parentFrame, "About JCards", true);

		ResourceBundle buttonLabels = JCards.getButtonsBundle();

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		setLayout(gbl);

		info = new TextArea(8, 30);
		info.setEditable(false);
		info.setBackground(Color.white);
		info.setForeground(Color.black);
		setAboutInfo();
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.insets = new Insets(5, 5, 5, 5);
		gbl.setConstraints(info, gbc);
		add(info);

		Button about = new Button(buttonLabels.getString("about"));
		about.setActionCommand("about");
		about.addActionListener(this);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.fill = GridBagConstraints.NONE;
		gbl.setConstraints(about, gbc);
		add(about);

		Button gc = new Button(buttonLabels.getString("gc"));
		gc.setActionCommand("gc");
		gc.addActionListener(this);
		gbc.weightx = 0.0;
		gbl.setConstraints(gc, gbc);
		add(gc);

		Button ok = new Button(buttonLabels.getString("ok"));
		ok.setActionCommand("ok");
		ok.addActionListener(this);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(ok, gbc);
		add(ok);
	}

	/** Show some info about the program */
	private void setAboutInfo()
	{
		StringBuffer sb = new StringBuffer("JCards ");
		sb.append(JCards.VERSION);
		sb.append("\nNick Sydenham");
		sb.append("\n<nsydenham@yahoo.co.uk>");
		sb.append("\nhttp://www.nsydenham.net/JCards.html");

		sb.append("\n\nAcknowledgements");
		sb.append("\n  Richard Carpenter");
		sb.append("\n  Dr Oliver Corff");
		sb.append("\n  Bill Curr");
		sb.append("\n  Daniel Durand");
		sb.append("\n  Mathis Heydtmann");
		sb.append("\n  Roy van der Kuil");
		sb.append("\n  Tibi Molnar");
		sb.append("\n  Johann N. Loefflmann");

		info.setText(sb.toString());
	}

	/** Set some useful system info */
	private void setSystemInfo()
	{
		StringBuffer sb = new StringBuffer(System.getProperty("java.vendor"));
		sb.append(" ");
		sb.append(System.getProperty("java.version"));
		sb.append("\n");
		sb.append(System.getProperty("os.name"));
		sb.append(" ");
		sb.append(System.getProperty("os.arch"));
		sb.append(" ");
		sb.append(System.getProperty("os.version"));
		sb.append("\n");
		sb.append("Username: ");
		sb.append(System.getProperty("user.name"));
		sb.append("\nHome dir: ");
		sb.append(System.getProperty("user.home"));
		sb.append("\nWorking dir: ");
		sb.append(System.getProperty("user.dir"));

		// memory
		Runtime rt = Runtime.getRuntime();
		long total, free, used;
		sb.append("\nTotal mem: ");
		total = rt.totalMemory() / 1024;
		sb.append(total);
		sb.append("KB\nFree mem: ");
		free = rt.freeMemory() / 1024;
		sb.append(free);
		sb.append("KB\nUsed mem: ");
		used = total - free;
		sb.append(used);
		sb.append("KB");

		info.setText(sb.toString());
		info.setCaretPosition(0);
	}

	/** User clicked OK */
	public void actionPerformed(ActionEvent e)
	{
		String ac = e.getActionCommand();

		if (ac.equals("ok")) {
			dispose();
		} else if (ac.equals("about")) {
			setAboutInfo();
		} else if (ac.equals("gc")) {
			System.gc();
			setSystemInfo();
		}
	}

	/** Dialog is displayed */
	public void show()
	{
		super.show();
		if (isShowing()) {
			info.setCaretPosition(0);
		}
	}
}
