package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

/*
 * JCards
 * Copyright (C) 2004 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Dialog used for exporting categories to a file
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CatExportDialog extends Dialog implements ActionListener, JCActionListener, WindowListener
{
	private JCards jcards = null;
	private TextField fileName = null;
	private Choice charSet, fieldDelimiter;
	
	/** Creates a new instance of CatExportDialog */
	public CatExportDialog(JCards jcards)
	{
		super(jcards, true);
		
		this.jcards = jcards;
		
		ResourceBundle buttons = JCards.getButtonsBundle();
		ResourceBundle labels = JCards.getLabelsBundle();
		setTitle(labels.getString("cat.export"));
		setFont(jcards.baseFont);
		addWindowListener(this);
		
		Panel inputPanel = new Panel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(5, 5, 5, 5);
		
		Label label = new Label(labels.getString("field.delimiter"));
        inputPanel.add(label, gbc);

		fieldDelimiter = new Choice();
		for (int i=0; i<CSVDialog.FIELD_DELIM.length; i++) {
			fieldDelimiter.add(CSVDialog.FIELD_DELIM[i]);
		}
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        inputPanel.add(fieldDelimiter, gbc);
		
		label = new Label(labels.getString("file"));
		gbc.gridwidth = 1;
		inputPanel.add(label, gbc);
		
		fileName = new TextField(10);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		inputPanel.add(fileName, gbc);
		
		Button select = new Button(buttons.getString("browse"));
		select.setActionCommand("browse");
		select.addActionListener(this);
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		inputPanel.add(select, gbc);
		
		label = new Label(labels.getString("charset"));
		gbc.gridwidth = 1;
        inputPanel.add(label, gbc);

		charSet = new Choice();
		for (int i=0; i<CSVDialog.CHAR_SETS.length; i++) {
			charSet.add(CSVDialog.CHAR_SETS[i]);
		}
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        inputPanel.add(charSet, gbc);
		
		Panel buttonPanel = new Panel(new FlowLayout(FlowLayout.RIGHT, 5, 5));
		
		JCButton cancelButton = new JCButton(buttons.getString("cancel"), JCards.getImage(this, UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		JCButton okButton = new JCButton(buttons.getString("ok"), JCards.getImage(this, UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		
		setLayout(new BorderLayout(5, 10));
		add(inputPanel, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.SOUTH);
	}

	/** User clicked a button */
	public void actionPerformed(ActionEvent e)
	{
		String ac = e.getActionCommand();
		if (ac.equals("cancel")) {
			dispose();
		} else if (ac.equals("browse")) {
			selectFile();
		} else if (ac.equals("ok")) {
			exportCategories();
		}
	}
	
	/** User clicked OK or Cancel */
	public void actionPerformed(JCActionEvent e)
	{
		actionPerformed(new ActionEvent(e.getSource(), e.getID(), e.getActionCommand()));
	}
	
	/** Export the categories to the selected file */
	private void exportCategories()
	{
		String file = fileName.getText();
		String cs = charSet.getSelectedItem();
		String fd = fieldDelimiter.getSelectedItem();
		if (file.length() > 0)
		{
			File f = new File(file);
			try
			{
				Hashtable cats = DBDef.getInstance().getCategories();
				PrintWriter print = new PrintWriter(new BufferedWriter(
					new OutputStreamWriter(new FileOutputStream(f), cs)));
				Vector items;
				String name;
				StringBuffer sb;
				Enumeration ienm;
				Enumeration enm = cats.keys();
				while (enm.hasMoreElements())
				{
					name = (String)enm.nextElement();
					sb = new StringBuffer(name);
					sb.append("=");
					items = (Vector)cats.get(name);
					ienm = items.elements();
					while (ienm.hasMoreElements())
					{
						sb.append(ienm.nextElement());
						sb.append(fd);
					}
					sb.setLength(sb.length()-1);
					print.println(sb.toString());
				}
				print.close();

				InfoDialog.showInfoDialog(jcards, jcards.getMessage("exported"), InfoDialog.DIALOG_INFO, jcards.getMessage("exported"));
				dispose();
			} catch (IOException ioex) {
				String[] msgLines = { jcards.getMessage("cannot.export.cats"), ioex.getMessage() };
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, msgLines);
			}
		}
		else
		{
			InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, jcards.getMessage("no.file"));
		}
	}
	
	/** Select a file to save to */
	private void selectFile()
	{
		FileDialog fDialog = new FileDialog(jcards, jcards.getMessage("file.select"), FileDialog.SAVE);
		fDialog.show();
		String file = fDialog.getFile();
		String dir = fDialog.getDirectory();
		fDialog.dispose();

		if (file != null)
		{
			file = dir + file;
			fileName.setText(file);
			fileName.setCaretPosition(file.length());
		}
	}
	
	/** Remove the dialog window */
	public void windowClosing(WindowEvent e)
	{
		dispose();
	}

	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}

}
