package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;

import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.dialog.InfoDialog;
import nrs.jcards.dialog.InputDialog;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Allows the user to edit categories
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CategoryDialog extends Dialog implements ActionListener, ItemListener, JCActionListener, WindowListener
{
	private JCards jcards = null;
	private Vector catItems = null;
	private ResourceBundle labels = JCards.getLabelsBundle();
	private Choice catNameList;
	private Label catLabel;
	private Panel buttonPanel;
	private java.awt.List catItemList;

	/** Creates new form CategoryDialog */
	public CategoryDialog(JCards jcards, boolean modal)
	{
		super(jcards, modal);

		this.jcards = jcards;

		setFont(jcards.baseFont);

		initComponents();

		// when user clicks close window decoration
		addWindowListener(this);
	}

	private void initComponents()
	{
		ResourceBundle buttons = JCards.getButtonsBundle();
		GridBagConstraints gridBagConstraints;

		catLabel = new Label(labels.getString("category")+":");
		catLabel.setFont(new Font("Dialog", 1, 12));

		catNameList = new Choice();
		catNameList.add("          ");
		catNameList.addItemListener(this);
		// add categories to list
		DBDef dbdef = DBDef.getInstance();
		Hashtable cats = dbdef.getCategories();
		if (cats != null)
		{
			Enumeration enm = cats.keys();
			while (enm.hasMoreElements())
			{
				catNameList.add((String)enm.nextElement());
			}
		}

		Button newButton = new Button(buttons.getString("new"));
		newButton.setActionCommand("new");
		newButton.addActionListener(this);

		Button renameButton = new Button(buttons.getString("rename"));
		renameButton.setActionCommand("rename");
		renameButton.addActionListener(this);

		Button removeButton = new Button(buttons.getString("remove"));
		removeButton.setActionCommand("remove");
		removeButton.addActionListener(this);

		catItemList = new List();
		catItemList.addActionListener(this);
		buttonPanel = new Panel();

		JCButton addButton = new JCButton(null, jcards.getImage(UserSettings.ICON_ADD), BWTEnum.STRING_LEFT);
		addButton.setActionCommand("add");
		addButton.addActionListener(this);

		JCButton editButton = new JCButton(null, JCards.getImage(this, UserSettings.ICON_EDIT), BWTEnum.STRING_LEFT);
		editButton.setActionCommand("edit");
		editButton.addActionListener(this);

		JCButton delButton = new JCButton(null, jcards.getImage(UserSettings.ICON_DELETE), BWTEnum.STRING_LEFT);
		delButton.setActionCommand("delete");
		delButton.addActionListener(this);

		JCButton upButton = new JCButton(null, jcards.getImage(UserSettings.ICON_UP), BWTEnum.STRING_LEFT);
		upButton.setActionCommand("up");
		upButton.addActionListener(this);

		JCButton downButton = new JCButton(null, jcards.getImage(UserSettings.ICON_DOWN), BWTEnum.STRING_LEFT);
		downButton.setActionCommand("down");
		downButton.addActionListener(this);

		JCButton okButton = new JCButton(null, jcards.getImage(UserSettings.ICON_OK), BWTEnum.STRING_LEFT);
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);

		setLayout(new GridBagLayout());

		setModal(true);
		setTitle(labels.getString("categories"));

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(catLabel, gridBagConstraints);

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(catNameList, gridBagConstraints);

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		gridBagConstraints.weightx = 1.0;
		add(newButton, gridBagConstraints);

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(renameButton, gridBagConstraints);

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(removeButton, gridBagConstraints);

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
		gridBagConstraints.gridheight = GridBagConstraints.RELATIVE;
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 1.0;
		gridBagConstraints.insets = new Insets(5, 2, 5, 2);
		add(catItemList, gridBagConstraints);

		buttonPanel.setLayout(new GridLayout(2, 3, 5, 0));

		buttonPanel.add(addButton);
		buttonPanel.add(editButton);
		buttonPanel.add(delButton);
		buttonPanel.add(upButton);
		buttonPanel.add(downButton);
		buttonPanel.add(okButton);

		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(buttonPanel, gridBagConstraints);

		pack();
	}

	/** @see #actionPerformed(ActionEvent) */
	public void actionPerformed(JCActionEvent e)
	{
		ActionEvent ae = new ActionEvent(e.getSource(), e.getID(), e.getActionCommand());
		actionPerformed(ae);
	}

	/** User clicked a button */
	public void actionPerformed(ActionEvent e)
	{
		Component comp = (Component)e.getSource();

		if (comp instanceof java.awt.List)
		{
			editItem();
		}
		else
		{
			String ac = e.getActionCommand();

			if (ac.equalsIgnoreCase("new"))	{
				newCategory();
			} else if (ac.equalsIgnoreCase("rename")) {
				renameCategory();
			} else if (ac.equalsIgnoreCase("remove")) {
				deleteCategory();
			} else if (ac.equalsIgnoreCase("add")) {
				addItem();
			} else if (ac.equalsIgnoreCase("delete")) {
				deleteItem();
			} else if (ac.equalsIgnoreCase("edit")) {
				editItem();
			} else if (ac.equalsIgnoreCase("up")) {
				moveItemUp();
			} else if (ac.equalsIgnoreCase("down")) {
				moveItemDown();
			} else if (ac.equalsIgnoreCase("ok")) {
				setVisible(false);
				dispose();
			}
		}
	}

	/** Add a new category list */
	private void newCategory()
	{
		String catName = InputDialog.showInputDialog(jcards, jcards.getMessage("new.category"), labels.getString("name"));
		catName = catName.trim(); // remove spaces
		if (catName.length() > 0)
		{
			catItems = new Vector();
			DBDef.getInstance().setCategory(catName, catItems);
			catNameList.add(catName);
			catNameList.select(catName);
			itemStateChanged(null);
		}
	}

	/** Rename a category */
	private void renameCategory()
	{
		String catName = catNameList.getSelectedItem().trim();
		if (catName.length() > 0)
		{
			String newName = InputDialog.showInputDialog(jcards, jcards.getMessage("new.name"), labels.getString("category"), catName);
			if (newName.length() > 0)
			{
				DBDef def = DBDef.getInstance();
				def.renameCategory(catName, newName);
				catNameList.remove(catName);
				catNameList.add(newName);
				catNameList.select(newName);
			}
		}
	}

	/** Delete a category list */
	private void deleteCategory()
	{
		String catName = catNameList.getSelectedItem().trim();
		if (catName.length() > 0)
		{
			int ret = InfoDialog.showInfoDialog(jcards, jcards.getMessage("confirm"), InfoDialog.DIALOG_CONFIRM, jcards.getMessage("are.you.sure"));
			if (ret == InfoDialog.YES)
			{
				if (DBDef.getInstance().removeCategory(catName))
				{
					catNameList.remove(catName);
					catItemList.removeAll();
				}
				else
				{
					InfoDialog.showInfoDialog(jcards, jcards.getMessage("cannot.delete"), InfoDialog.DIALOG_INFO, jcards.getMessage("field.dep"));
				}
			}
		}
	}

	/** Add an item to a category list */
	private void addItem()
	{
		String catName = catNameList.getSelectedItem().trim();
		if (catName.length() > 0)
		{
			ResourceBundle labels = JCards.getLabelsBundle();
			String item = InputDialog.showInputDialog(jcards, jcards.getMessage("item.new"), labels.getString("item"));
			if (item.length() > 0)
			{
				DBDef.getInstance().addCategoryItem(catNameList.getSelectedItem(), item);
				catItemList.add(item);
			}
		}
		else
		{
			InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, jcards.getMessage("select.category"));
		}
	}

	/** Delete an item from a category list */
	private void deleteItem()
	{
		String item = catItemList.getSelectedItem();
		if (item != null)
		{
			DBDef.getInstance().removeCategoryItem(catNameList.getSelectedItem(), item);
			catItemList.remove(item);
		}
	}

	/** Edit a category item */
	private void editItem()
	{
		String item = catItemList.getSelectedItem();
		if (item != null)
		{
			int index = catItemList.getSelectedIndex();
			ResourceBundle labels = JCards.getLabelsBundle();
			String ret = InputDialog.showInputDialog(jcards, jcards.getMessage("item.edit"), labels.getString("item"), item);
			if (ret.length() > 0)
			{
				DBDef.getInstance().renameCategoryItem(catNameList.getSelectedItem(), item, ret);
				catItemList.remove(item);
				catItemList.add(ret, index);
				catItemList.select(index);
			}
		}
	}

	/** Move a category item up one place */
	private void moveItemUp()
	{
		String item = catItemList.getSelectedItem();
		if (item != null)
		{
			int pos = catItemList.getSelectedIndex();
			if (pos > 0)
			{
				DBDef.getInstance().moveCategoryItem(catNameList.getSelectedItem(), item, -1);

				catItemList.remove(pos);
				catItemList.add(item, pos-1);
				catItemList.select(pos-1);
			}
		}
	}

	/** Move a category item down one place */
	private void moveItemDown()
	{
		String item = catItemList.getSelectedItem();
		if (item != null)
		{
			int pos = catItems.indexOf(item);
			if (pos < catItems.size()-1)
			{
				DBDef.getInstance().moveCategoryItem(catNameList.getSelectedItem(), item, 1);

				catItemList.remove(pos);
				catItemList.add(item, pos+1);

				catItemList.select(pos+1);
			}
		}
	}

	/**
	 * User selected a category name
	 */
	public void itemStateChanged(ItemEvent e)
	{
		String catName = catNameList.getSelectedItem();

		if (catName.length() > 0)
		{
			catItems = (Vector)DBDef.getInstance().getCategory(catName);
			if (catItems != null)
			{
				catItemList.removeAll();
				Enumeration enm = catItems.elements();
				while (enm.hasMoreElements())
				{
					catItemList.add((String)enm.nextElement());
				}
			}
		}
		else
		{
			catItemList.removeAll();
		}
	}

	/**
	 * Get the maximum size of the dialog
	 * @return max size
	 */
	public Dimension getMaximumSize()
	{
		return jcards.getMaximumSize();
	}

	/**
	 * Get the preferred size of the dialog
	 * @return preferred size
	 */
	public Dimension getPreferredSize()
	{
		if (jcards.getUserSettings().isPda())
		{
			return jcards.getPreferredSize();
		} else
		{
			return new Dimension(300, 300);
		}
	}

	/** User wants to close the window */
	public void windowClosing(WindowEvent e)
	{
		dispose();
	}

	public void windowOpened(WindowEvent e)
	{}
	public void windowActivated(WindowEvent e)
	{}
	public void windowClosed(WindowEvent e)
	{}
	public void windowIconified(WindowEvent e)
	{}
	public void windowDeiconified(WindowEvent e)
	{}
	public void windowDeactivated(WindowEvent e)
	{}

}
