package nrs.jcards.dialog;

import java.awt.*;
import java.util.ResourceBundle;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;
import nrs.jcards.Cards;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

/*
 * JCards
 * Copyright (C) 2004 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Dialog for editing the comment associated with a database
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CommentDialog extends Dialog implements JCActionListener
{
	private TextArea comment;
	private Cards cards = null;
	
	/** Creates a new instance of CommentDialog */
	public CommentDialog(JCards jcards, Cards cards)
	{
		super(jcards, true);
		
		this.cards = cards;
		
		ResourceBundle labels = JCards.getLabelsBundle();
		setTitle(labels.getString("edit.comment"));
		setFont(jcards.baseFont);
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		
		setLayout(gbl);
		
		Insets ins = new Insets(2, 2, 2, 2);
		gbc.insets = ins;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		
		comment = new TextArea(cards.getInfo(), 5, 20);
		gbl.setConstraints(comment, gbc);
		add(comment);

		ResourceBundle buttons = JCards.getButtonsBundle();
		JCButton okButton = new JCButton(buttons.getString("ok"), jcards.getImage(UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		okButton.setActionCommand("OK");
		okButton.addActionListener(this);
		gbc.gridwidth = 1;
		gbc.weighty = 0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbl.setConstraints(okButton, gbc);
		add(okButton);
		
		JCButton cancel = new JCButton(buttons.getString("cancel"), jcards.getImage(UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancel.setActionCommand("CANCEL");
		cancel.addActionListener(this);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(cancel, gbc);
		add(cancel);
	}
	
	/** User clicked a button. If they clicked OK the comment is set. */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();
		
		if (ac.equals("OK")) {
			cards.setInfo(comment.getText());
		}
		
		dispose();
	}
}
