package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;
import nrs.jcards.Counter;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Dialog for editing counters
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CounterDialog extends Dialog implements ItemListener, JCActionListener, WindowListener
{
	private JCards jcards;
	private DBDef def = DBDef.getInstance();
	private Choice counterChoice;
	private TextField value, increment;

	/** Creates a new instance of CounterDialog */
	public CounterDialog(JCards jcards)
	{
		super(jcards, true);

		this.jcards = jcards;

		ResourceBundle labels = JCards.getLabelsBundle();
		setTitle(labels.getString("counters"));
		setFont(jcards.baseFont);

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		setLayout(gbl);

		gbc.insets = new Insets(2, 2, 2, 2);
		gbc.anchor = GridBagConstraints.WEST;

		addLabel(labels.getString("counters"), gbc, gbl);
		counterChoice = new Choice();
		counterChoice.add("");
		Hashtable counters = def.getCounters();
		Counter c;
		Enumeration enm = counters.elements();
		while (enm.hasMoreElements())
		{
			c = (Counter)enm.nextElement();
			counterChoice.add(c.getName());
		}
		counterChoice.addItemListener(this);
		addComponent(counterChoice, gbc, gbl);

		addLabel(labels.getString("value"), gbc, gbl);
		value = new TextField(20);
		addComponent(value, gbc, gbl);

		addLabel(labels.getString("increment"), gbc, gbl);
		increment = new TextField(20);
		addComponent(increment, gbc, gbl);

		Panel buttonPanel;
		if (jcards.getUserSettings().isPda()) {
			buttonPanel = new Panel(new GridLayout(2, 2, 2, 2));
		} else {
			buttonPanel = new Panel(new GridLayout(1, 4, 2, 2));
		}

		ResourceBundle buttons = JCards.getButtonsBundle();
		JCButton addButton = new JCButton(buttons.getString("new"), jcards.getImage(UserSettings.ICON_ADD), BWTEnum.STRING_RIGHT);
		addButton.setActionCommand("new");
		addButton.addActionListener(this);
		buttonPanel.add(addButton);

		JCButton deleteButton = new JCButton(buttons.getString("delete"), jcards.getImage(UserSettings.ICON_DELETE), BWTEnum.STRING_RIGHT);
		deleteButton.setActionCommand("delete");
		deleteButton.addActionListener(this);
		buttonPanel.add(deleteButton);

		JCButton updateButton = new JCButton(buttons.getString("update"), jcards.getImage(UserSettings.ICON_SAVE), BWTEnum.STRING_RIGHT);
		updateButton.setActionCommand("update");
		updateButton.addActionListener(this);
		buttonPanel.add(updateButton);

		JCButton okButton = new JCButton(buttons.getString("ok"), jcards.getImage(UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);

		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		gbl.setConstraints(buttonPanel, gbc);
		add(buttonPanel);

		// listen for close events
		addWindowListener(this);
	}

	/** Utility method for setting GridBagConstraints on a label */
	private void addLabel(String labelText, GridBagConstraints gbc, GridBagLayout gbl)
	{
		Label l = new Label(labelText);
		gbc.gridwidth = 1;
		gbc.weightx = 0.0;
		gbc.fill = GridBagConstraints.NONE;
		gbl.setConstraints(l, gbc);
		add(l);
	}

	/** Utility method for setting GridBagConstraints on a component */
	private void addComponent(Component c, GridBagConstraints gbc, GridBagLayout gbl)
	{
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.weightx = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbl.setConstraints(c, gbc);
		add(c);
	}

	/** Button was clicked */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();

		if (ac.equals("new"))
		{
			String newCounter = InputDialog.showInputDialog(jcards, jcards.getMessage("new.counter"), jcards.getMessage("new.counter.name"));
			if (newCounter != null && newCounter.length() > 0)
			{
				Counter c = new Counter(newCounter, 0, 1);
				def.addCounter(c);
				counterChoice.add(newCounter);
				counterChoice.select(newCounter);
				value.setText("0");
				increment.setText("1");
			}
		}
		else if (ac.equals("delete"))
		{
			if (InfoDialog.showInfoDialog(jcards, jcards.getMessage("confirm.delete"), InfoDialog.DIALOG_CONFIRM, jcards.getMessage("are.you.sure")) != InfoDialog.YES) {
				return;
			}

			String csel = counterChoice.getSelectedItem();
			if (csel != null && csel.length() > 0)
			{
				String fName = def.removeCounter(csel);
				if (fName == null)
				{
					counterChoice.remove(csel);
					counterChoice.select(0);
					value.setText("0");
					increment.setText("1");
				}
				else
				{
					MessageFormat msgFormat = new MessageFormat(jcards.getMessage("counter.in.use"));
					InfoDialog.showInfoDialog(jcards, jcards.getMessage("cannot.delete"), InfoDialog.DIALOG_INFO, msgFormat.format(fName));
				}
			}
		}
		else if (ac.equals("update"))
		{
			updateCounter();
		}
		else if (ac.equals("ok"))
		{
			windowClosing(null);
		}
	}

	/**
	 * Update a counter
	 * @return true if the counter was updated, false if there is a problem
	 */
	private boolean updateCounter()
	{
		String csel = counterChoice.getSelectedItem();
		if (csel != null && csel.length() > 0)
		{
			String val = value.getText();
			String incr = increment.getText();

			// error checking
			if (val.length() == 0) {
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, jcards.getMessage("null.value"));
				value.requestFocus();
				return false;
			}
			if (incr.length() == 0) {
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, jcards.getMessage("null.increment"));
				increment.requestFocus();
				return false;
			}

			double vall, incrl;
			try {
				vall = Double.valueOf(val).doubleValue();
				incrl = Double.valueOf(incr).doubleValue();
			} catch (NumberFormatException nfex) {
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, jcards.getMessage("val.incr.num"));
				return false;
			}

			if (incrl == 0.0) {
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("error"), InfoDialog.DIALOG_INFO, jcards.getMessage("zero.increment"));
				increment.requestFocus();
				return false;
			}

			Counter c = new Counter(csel, vall, incrl);
			def.updateCounter(c);

			ResourceBundle labels = JCards.getLabelsBundle();
			InfoDialog.showInfoDialog(jcards, labels.getString("ok"), InfoDialog.DIALOG_INFO, jcards.getMessage("counter.updated"));

			return true;
		}

		return false;
	}

	/** User selected a different Counter */
	public void itemStateChanged(ItemEvent e)
	{
		String counterName = counterChoice.getSelectedItem();
		if (counterName.length() > 0)
		{
			Counter counter = def.getCounter(counterName);
			if (counter != null)
			{
				value.setText(Double.toString(counter.getValue()));
				increment.setText(Double.toString(counter.getIncrement()));
			}
		}
		else
		{
			value.setText("0");
			increment.setText("1");
		}
	}

	/** Close the dialog */
	public void windowClosing(WindowEvent e)
	{
		hide();
		dispose();
	}

	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
}
