package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.NoSuchElementException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;

import nrs.jcards.DBDef;
import nrs.jcards.FieldDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.util.PostfixCalculator;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * This class displays the fields and allows additions, modifications and deletion
 * of fields.
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class EditFieldDialog extends Dialog implements JCActionListener, ItemListener
{
	private JCards jcards = null;
	private FieldsDialog fieldsDialog = null;
	private int fieldIndex = -1;

	private Checkbox manCB = null;
	private Choice catName = null;
	private Choice countName = null;
	private Choice memoLineCh = null;
	private TextField fieldName = null;
	private TextField defaultValue = null;
	private Choice fieldType = null;

	/** Creates a new instance of EditFieldDialog */
	public EditFieldDialog(JCards jcards, FieldsDialog fDialog, String title, boolean edit, int fieldIndex)
	{
		super(jcards, title, true);

		this.jcards = jcards;
		fieldsDialog = fDialog;
		this.fieldIndex = fieldIndex;

		ResourceBundle buttons = JCards.getButtonsBundle();
		ResourceBundle labels = JCards.getLabelsBundle();

		Label l;
		FieldDef fdef = null;
		if (edit) {
			fdef = DBDef.getInstance().getFieldDef(fieldIndex);
		}
		DBDef def = DBDef.getInstance();

		setFont(jcards.baseFont);
		setLayout(new BorderLayout());

		// add toolbar
		Panel toolbar = new Panel(new FlowLayout(FlowLayout.LEADING, 1, 0));
		JCButton ok = new JCButton(buttons.getString("ok"), jcards.getImage(UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		if (edit) ok.setActionCommand("edit_OK");
		else ok.setActionCommand("new_OK");
		ok.addActionListener(this);
		toolbar.add(ok);

		JCButton cancel = new JCButton(buttons.getString("cancel"), jcards.getImage(UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancel.setActionCommand("cancel");
		cancel.addActionListener(this);
		toolbar.add(cancel);

		add(toolbar, BorderLayout.NORTH);

		// add fields
		Panel input = new Panel(new GridLayout(7, 2, 2, 2));
		input.add(new Label(labels.getString("name")+":"));
		fieldName = new TextField(10);
		input.add(fieldName);

		input.add(new Label(labels.getString("type")+":"));
		fieldType = new Choice();
		boolean addedCategory = false;
		boolean addedCounter = false;
		boolean addedCalculated = false;
		String[] types = FieldDef.getTypes();
		if (edit)
		{
			String fType = FieldDef.getTypeName(fdef.getType());
			String defFType;
			for (int i=0; i<types.length; i++)
			{
				defFType = types[i];
				if (defFType.equals(fType) || defFType.equals(FieldDef.STRING_NAME)
					|| defFType.equals(FieldDef.MEMO_NAME))
				{
					fieldType.add(defFType);
					if (defFType.equals(FieldDef.CATEGORY_NAME)) {
						addedCategory = true;
					} else if (defFType.equals(FieldDef.COUNTER_NAME)) {
						addedCounter = true;
					} else if (defFType.equals(FieldDef.CALCULATED_NAME)) {
						addedCalculated = true;
					}
				}
			}
		}
		else
		{
			for (int i=0; i<types.length; i++)	{
				fieldType.add(types[i]);
			}
			addedCategory = addedCounter = addedCalculated = true;
		}

		// only allow Categories and Counters if one or more have been defined
		if (addedCategory && def.getCategories().size() == 0)
		{
			fieldType.remove(FieldDef.CATEGORY_NAME);
		}
		if (addedCounter && def.getCounters().size() == 0)
		{
			fieldType.remove(FieldDef.COUNTER_NAME);
		}

		// only allow Calculated if other numeric field defined
		if (addedCalculated && !def.areCalculationsAllowed())
		{
			fieldType.remove(FieldDef.CALCULATED_NAME);
		}

		fieldType.addItemListener(this);
		input.add(fieldType);

		input.add(new Label(labels.getString("category")+":"));
		catName = new Choice();
		catName.add("");
		// add categories
		Hashtable cats = def.getCategories();
		Enumeration enm = cats.keys();
		while (enm.hasMoreElements())
		{
			catName.add((String)enm.nextElement());
		}
		catName.setEnabled(false);
		input.add(catName);

		input.add(new Label(labels.getString("counter")+":"));
		countName = new Choice();
		countName.add("");
		// add counters
		Hashtable counts = def.getCounters();
		enm = counts.keys();
		while (enm.hasMoreElements())
		{
			countName.add((String)enm.nextElement());
		}
		countName.setEnabled(false);
		input.add(countName);

		input.add(new Label(labels.getString("default")+": "));
		defaultValue = new TextField(10);
		input.add(defaultValue);

		input.add(new Label(labels.getString("mandatory")+":"));
		manCB = new Checkbox();
		input.add(manCB);

		input.add(new Label(labels.getString("memo.lines")+":"));
		memoLineCh = new Choice();
		for (int i=2; i<20; i++)
		{
			memoLineCh.add(Integer.toString(i));
		}
		memoLineCh.setEnabled(false);
		input.add(memoLineCh);

		// fill in the fields if editing
		if (edit)
		{
			fieldName.setText(fdef.getName());
			fieldType.select(FieldDef.getTypeName(fdef.getType()));
			switch (fdef.getType())
			{
				case FieldDef.CATEGORY:
					catName.select(fdef.getCategory());
					catName.setEnabled(true);
					break;

				case FieldDef.COUNTER:
					defaultValue.setEnabled(false);
					defaultValue.setText(null);
					countName.select(fdef.getCounter());
					countName.setEnabled(true);
					break;

				case FieldDef.MEMO:
					memoLineCh.select(fdef.getMemoLines()-2);
					memoLineCh.setEnabled(true);
					break;
			}
			manCB.setState(fdef.isMandatory());

			String defVal = fdef.getDefaultValue();
			if (defVal != null) {
				defaultValue.setText(defVal);
			}
		}
		add(input, BorderLayout.CENTER);
	}

	/** Set focus to field name */
	public void show()
	{
		super.show();
		fieldName.requestFocus();
	}

	/** User clicked a button */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();

		if (ac.equals("new_OK")) updateField(true);
		else if (ac.equals("edit_OK")) updateField(false);
		else if (ac.equals("cancel")) dispose();
	}

	/** Method called when item in type choice list is changed */
	public void itemStateChanged(ItemEvent e)
	{
		String type = fieldType.getSelectedItem();

		if (type != null)
		{
			manCB.setEnabled(true);
			catName.setEnabled(false);
			countName.setEnabled(false);
			memoLineCh.setEnabled(false);
			defaultValue.setEnabled(true);

			if (type.equals(FieldDef.BOOLEAN_NAME))
			{
				// boolean fields cannot be mandatory
				manCB.setState(false);
				manCB.setEnabled(false);
			}
			else if (type.equals(FieldDef.CATEGORY_NAME))
			{
				catName.setEnabled(true);
				catName.requestFocus();
			}
			else if (type.equals(FieldDef.COUNTER_NAME))
			{
				countName.setEnabled(true);
				defaultValue.setEnabled(false);
				defaultValue.setText(null);
				manCB.setState(false);
				manCB.setEnabled(false);
				countName.requestFocus();
			}
			else if (type.equals(FieldDef.CALCULATED_NAME))
			{
				manCB.setState(false);
				manCB.setEnabled(false);
				defaultValue.requestFocus();
			}
			else if (type.equals(FieldDef.MEMO_NAME))
			{
				memoLineCh.setEnabled(true);
				memoLineCh.requestFocus();
			}
		}
	}

	/**
	 * Create or update a field
	 * @param newField if true create a new field, otherwise edit an existing field
	 */
	private void updateField(boolean newField)
	{
		DBDef def = DBDef.getInstance();
		String defaultText;

		String fName = fieldName.getText();
		if (fName == null || fName.length() == 0)
		{
			Toolkit.getDefaultToolkit().beep();
			InfoDialog.showInfoDialog(jcards, jcards.getMessage("null.entry"), InfoDialog.DIALOG_INFO, jcards.getMessage("field.name.null"));
			return;
		}

		String fTypeS = fieldType.getSelectedItem();
		int fType = FieldDef.getType(fTypeS);
		// first field cannot be a boolean
		if (fType == FieldDef.BOOLEAN && def.getNumberOfFields() ==0)
		{
			Toolkit.getDefaultToolkit().beep();
			InfoDialog.showInfoDialog(jcards, jcards.getMessage("cannot.create.field"), InfoDialog.DIALOG_INFO, jcards.getMessage("boolean.first"));
			return;
		}
		FieldDef fieldDef = new FieldDef(fType, fName);

		if (fType == FieldDef.CATEGORY)
		{
			String cat = catName.getSelectedItem();
			if (cat == null || cat.length() == 0)
			{
				Toolkit.getDefaultToolkit().beep();
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("null.category"), InfoDialog.DIALOG_INFO, jcards.getMessage("select.category"));
				catName.requestFocus();
				return;
			}
			fieldDef.setCategory(cat);
		}
		else if (fType == FieldDef.COUNTER)
		{
			String count = countName.getSelectedItem();
			if (count == null || count.length() == 0)
			{
				Toolkit.getDefaultToolkit().beep();
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("null.counter"), InfoDialog.DIALOG_INFO, jcards.getMessage("select.counter"));
				countName.requestFocus();
				return;
			}
			fieldDef.setCounter(countName.getSelectedItem());
		}
		else if (fType == FieldDef.CALCULATED)
		{
			defaultText = defaultValue.getText();
			if (defaultText.length() == 0) {
				Toolkit.getDefaultToolkit().beep();
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("no.calc"), InfoDialog.DIALOG_INFO, jcards.getMessage("enter.calc"));
				defaultValue.requestFocus();
				return;
			}

			String msg = checkCalculation(defaultText);
			if (msg != null) {
				Toolkit.getDefaultToolkit().beep();
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("incorrect.calc"), InfoDialog.DIALOG_INFO, msg);
				defaultValue.requestFocus();
				return;
			}
			else
			{
				fieldDef.setDefaultValue(defaultText);
			}
		}
		else if (fType == FieldDef.MEMO)
		{
			fieldDef.setMemoLines(Integer.parseInt(memoLineCh.getSelectedItem()));
		}

		// first field is always mandatory
		if (def.getNumberOfFields() == 0 || (!newField && fieldIndex == 0)) {
			fieldDef.setMandatory(true);
		} else {
			fieldDef.setMandatory(manCB.getState());
		}

		// set the optional default
		defaultText = defaultValue.getText();
		if (defaultText.length() > 0 && fType != FieldDef.COUNTER)
		{
			if (!fieldDef.setDefaultValue(defaultText)) {
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("incorrect.default"), InfoDialog.DIALOG_INFO, jcards.getMessage("default.type.mismatch"));
				defaultValue.requestFocus();
				return;
			}
		}

		if (newField)
		{
			if (def.addFieldDef(fieldDef)) {
				fieldsDialog.selectField(-1);
			} else {
				Toolkit.getDefaultToolkit().beep();
				InfoDialog.showInfoDialog(jcards, jcards.getMessage("not.unique"), InfoDialog.DIALOG_INFO, jcards.getMessage("field.already.exists"));
				fieldName.requestFocus();
				return;
			}
		}
		else
		{
			def.replaceFieldDef(fieldIndex, fieldDef);
			fieldsDialog.selectField(fieldIndex);
		}
		dispose();
	}

	/**
	 * Check that the calculation is valid
	 * @param calculation the calculation to check
	 * @return null if the calculation is OK, or a message indicating the error
	 */
	private String checkCalculation(String calculation)
	{
		DBDef def = DBDef.getInstance();
		String token;
		FieldDef fd;
		StringTokenizer st = new StringTokenizer(calculation, " ");

		try
		{
			while (st.hasMoreTokens())
			{
				token = st.nextToken();

				// field
				if (token.startsWith("'"))
				{
					while (!token.endsWith("'")) {
						token += " ";
						token += st.nextToken();
					};
					// remove quotes
					token = token.substring(1, token.length()-1);

					// check field exists
					fd = def.getFieldDef(token);
					if (fd == null) {
						return formatMessage("field.no.exist", token);
					}
					// check field is of the right type
					if (!(fd.getType() == FieldDef.INTEGER || fd.getType() == FieldDef.DOUBLE ||
						fd.getType() == FieldDef.CALCULATED || fd.getType() == FieldDef.COUNTER))
					{
						return formatMessage("field.no.num", token);
					}
				}
				else if (PostfixCalculator.isOperand(token)) {}
				else if (isNumber(token)) {}
				else if (PostfixCalculator.isMathFunction(token)) {}
				else {
					return formatMessage("field.unknown", token);
				}
			}
		} catch (NoSuchElementException nsex) {
			return jcards.getMessage("field.end");
		}

		return null;
	}

	/** Format a message */
	private String formatMessage(String key, String token)
	{
		MessageFormat msgFormat = new MessageFormat(jcards.getMessage(key));
		return msgFormat.format(token);
	}

	/**
	 * Checks to see if a String is a number
	 * @param num the number to check
	 * @return true if number, false otherwise
	 */
	private final static boolean isNumber(String num)
	{
		try {
			Double.valueOf(num);
			return true;
		} catch (NumberFormatException ex) {}

		return false;
	}
}
