package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import nrs.jcards.DBDef;
import nrs.jcards.FieldDef;
import nrs.jcards.FieldsList;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

import jclass.bwt.*;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * This class displays the fields and allows additions, modifications and deletion
 * of fields.
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FieldsDialog extends Dialog implements JCActionListener, WindowListener
{
	private JCards jcards = null;
	private FieldsList fieldList = null;
	private int fieldIndex = -1;
	private boolean movedField = false;
	
	/** Initialise the FieldsDialog
	 * @param jcards JCards
	 */
	public FieldsDialog(JCards jcards)
	{
		super(jcards, true);
		
		this.jcards = jcards;
		
		ResourceBundle labels = JCards.getLabelsBundle();
		setTitle(labels.getString("edit.fields"));
		setFont(jcards.baseFont);
		setLayout(new BorderLayout());
		
		// add toolbar
		Panel toolbar = new Panel(new GridLayout(6, 1, 2, 2));
		
		JCButton ok = new JCButton(null, jcards.getImage(UserSettings.ICON_OK), BWTEnum.STRING_LEFT);
		ok.setActionCommand("Field_OK");
		ok.addActionListener(this);
		toolbar.add(ok);
		
		JCButton newField = new JCButton(null, jcards.getImage(UserSettings.ICON_ADD), BWTEnum.STRING_LEFT);
		newField.setActionCommand("Field_New");
		newField.addActionListener(this);
		toolbar.add(newField);
		
		JCButton editField = new JCButton(null, jcards.getImage(UserSettings.ICON_EDIT), BWTEnum.STRING_LEFT);
		editField.setActionCommand("Field_Edit");
		editField.addActionListener(this);
		toolbar.add(editField);
		
		JCButton deleteField = new JCButton(null, jcards.getImage(UserSettings.ICON_DELETE), BWTEnum.STRING_LEFT);
		deleteField.setActionCommand("Field_Delete");
		deleteField.addActionListener(this);
		toolbar.add(deleteField);
		
		JCButton up = new JCButton(null, jcards.getImage(UserSettings.ICON_UP), BWTEnum.STRING_LEFT);
		up.setActionCommand("Field_Up");
		up.addActionListener(this);
		toolbar.add(up);
		
		JCButton down = new JCButton(null, jcards.getImage(UserSettings.ICON_DOWN), BWTEnum.STRING_LEFT);
		down.setActionCommand("Field_Down");
		down.addActionListener(this);
		toolbar.add(down);
		
		add(toolbar, BorderLayout.EAST);
		
		fieldList = new FieldsList(jcards);
		DBDef.getInstance().addObserver(fieldList);
		fieldList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ResourceBundle labels = JCards.getLabelsBundle();
				showFieldDialog(labels.getString("edit.field"), true);
			}
		});
		add(fieldList, BorderLayout.CENTER);
		
		// when user clicks close window decoration
		addWindowListener(this);
	}
	
	/** Called when a button is clicked */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();
		ResourceBundle labels = JCards.getLabelsBundle();
		
		if (ac.equals("Field_OK"))
		{
			if (movedField)	{
				DBDef.getInstance().moveFieldFinished();
			}
			setVisible(false);
			dispose();
		}
		else if (ac.equals("Field_New")) showFieldDialog(labels.getString("add.field"), false);
		else if (ac.equals("Field_Edit"))
		{
			int sel = fieldList.getSelectedIndex();
			if (sel < 0) return;
			fieldIndex = sel;
			showFieldDialog(labels.getString("edit.field"), true);
		}
		else if (ac.equals("Field_Up")) fieldMove(-1);
		else if (ac.equals("Field_Down")) fieldMove(1);
		else if (ac.equals("Field_Delete")) deleteField();
	}
	
	/**
	 * Show the edit field dialog
	 * @param title title of the dialog
	 * @param edit true if editing
	 */
	private void showFieldDialog(String title, boolean edit)
	{
		int fieldIndex = fieldList.getSelectedIndex();
		EditFieldDialog efd = new EditFieldDialog(jcards, this, title, edit, fieldIndex);
		efd.pack();
		jcards.centreWindow(this, efd);
		efd.show();
	}
	
	/** Delete a field */
	private void deleteField()
	{
		DBDef dbdef = DBDef.getInstance();
		
		int selection = fieldList.getSelectedIndex();
		if (selection > -1 && dbdef.getNumberOfFields() > 1)
		{
			// is the user sure?
			int ret = InfoDialog.showInfoDialog(jcards, jcards.getMessage("confirm"), InfoDialog.DIALOG_CONFIRM, jcards.getMessage("are.you.sure"));
			if (ret == InfoDialog.YES)
			{
				/* if the next field is a boolean
				   and this field is the first then
				   disallow the operation as boolean
				   fields cannot be the first field */
				if (selection == 0)
				{
					FieldDef fd = dbdef.getFieldDef(1);
					if (fd != null && fd.getType() == FieldDef.BOOLEAN)
					{
						Toolkit.getDefaultToolkit().beep();
						InfoDialog.showInfoDialog(jcards, jcards.getMessage("cannot.delete"), InfoDialog.DIALOG_INFO, jcards.getMessage("cannot.delete.msg"));
						return;
					}
				}
				
				dbdef.removeFieldDef(selection);
			}
		}
	}
	
	/**
	 * Move a field up or down in the list
	 * @param incr either +1 or -1
	 */
	private void fieldMove(int incr)
	{
		int selection = fieldList.getSelectedIndex();
		DBDef dbdef = DBDef.getInstance();
		
		if (selection > -1)
		{
			// cannot move a boolean field to first in the list
			if ((dbdef.getNumberOfFields() > 1) && 
				(selection == 0 && incr == 1) || (selection == 1 && incr == -1))
			{
				FieldDef fd = dbdef.getFieldDef(1);
				
				if (fd.getType() == FieldDef.BOOLEAN)
				{
					Toolkit.getDefaultToolkit().beep();
					InfoDialog.showInfoDialog(jcards, jcards.getMessage("cannot.move"), InfoDialog.DIALOG_INFO, jcards.getMessage("cannot.move.msg"));
					return;
				}
			}
			int newpos = dbdef.moveField(selection, incr);
			fieldList.select(newpos);
			movedField = true;
		}
	}
	
	/**
	 * Select a field and make it visible
	 * @param fieldIndex index of the field to select. If set to -1 select the last item in the list
	 */
	void selectField(int fieldIndex)
	{
		if (fieldIndex == -1) {
			fieldIndex = fieldList.getItemCount()-1;
		}
		
		fieldList.select(fieldIndex);
		fieldList.makeVisible(fieldIndex);
	}
	
	/**
	 * Get the maximum size of the dialog
	 * @return max size
	 */
	public Dimension getMaximumSize()
	{
		return jcards.getMaximumSize();
	}
	
	/**
	 * Get the preferred size of the FieldsDialog
	 * @return preferred size
	 */
	public Dimension getPreferredSize()
	{
		if (jcards.getUserSettings().isPda()) {
			return jcards.getPreferredSize();
		} else {
			return new Dimension(400, 300);
		}
	}
	
	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
	
	/** User clicked close window decoration */
	public void windowClosing(WindowEvent e) {
		dispose();
	}
}