package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import nrs.jcards.DBDef;
import nrs.jcards.JCards;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Dialog used for searching
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FindDialog extends Dialog implements ActionListener, KeyListener, WindowListener
{
	private JCards jcards = null;
	private TextField findText = null;
	private Choice fields = null;
	private Checkbox matchCase = null, reFind = null;

	/**
	 * Creates a new instance of FindDialog
	 * @param jcards owning Frame
	 * @param searchText previous search text if set
	 * @param field previous search field if set
	 * @param mc previous setting for match case
	 */
	public FindDialog(JCards jcards, String searchText, String field, boolean mc)
	{
		super((Frame)jcards, true);

		this.jcards = jcards;

		ResourceBundle labels = JCards.getLabelsBundle();
		ResourceBundle buttonNames = JCards.getButtonsBundle();
		ResourceBundle menuNames = JCards.getMenuBundle();

		setFont(jcards.baseFont);
		setTitle(labels.getString("find"));

		setLayout(new GridLayout(5, 2, 2, 2));

		add(new Label(labels.getString("find")+":"));
		findText = new TextField(12);
		if (searchText != null)	{
			findText.setText(searchText);
		}
		findText.addKeyListener(this); // listen for enter & escape
		add(findText);

		add(new Label(labels.getString("field")));
		fields = new Choice();
		fields.add(menuNames.getString("all"));
		Vector fieldNames = DBDef.getInstance().getFieldNames();
		Enumeration enm = fieldNames.elements();
		while (enm.hasMoreElements()) {
			fields.add((String)enm.nextElement());
		}
		if (field != null) {
			fields.select(field);
		}

		add(fields);

		add(new Label(labels.getString("match.case")));
		matchCase = new Checkbox();
		matchCase.setState(mc);
		add(matchCase);

		add(new Label(labels.getString("find.in.results")));
		reFind = new Checkbox();
		add(reFind);

		Button search = new Button(buttonNames.getString("search"));
		search.setActionCommand("search");
		search.addActionListener(this);
		add(search);

		Button cancel = new Button(buttonNames.getString("cancel"));
		cancel.setActionCommand("cancel");
		cancel.addActionListener(this);
		add(cancel);

		// listen for close events
		addWindowListener(this);
	}

	/** User clicked a button */
	public void actionPerformed(ActionEvent actionEvent)
	{
		if (actionEvent.getActionCommand().equals("search"))
		{
			String text = findText.getText();
			if (text.length() == 0) return;

			// if no cards are found offer to search again
			if (!jcards.findCards(text, fields.getSelectedItem(), matchCase.getState(), reFind.getState()))
			{
				if (InfoDialog.showInfoDialog(jcards, jcards.getMessage("no.cards.found"),
						InfoDialog.DIALOG_CONFIRM, jcards.getMessage("repeat.search")) == InfoDialog.YES)
				{
					return;
				}
			}
		}

		setVisible(false);
		dispose();
	}

	/** Invoked when the user attempts to close the window
	 * from the window's system menu.  If the program does not
	 * explicitly hide or dispose the window while processing
	 * this event, the window close operation will be cancelled. */
	public void windowClosing(WindowEvent e)
	{
		dispose();
	}

	public void windowClosed(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}

	public void keyPressed(KeyEvent e) {}

	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			ActionEvent ae = new ActionEvent(this, 0, "search");
			actionPerformed(ae);
		}
		else if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			ActionEvent ae = new ActionEvent(this, 0, "cancel");
			actionPerformed(ae);
		}
	}

	public void keyTyped(KeyEvent e) {}

}
