package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.ResourceBundle;

import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

import jclass.bwt.*;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Allows the user to edit categories
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class InputDialog extends Dialog implements JCActionListener, KeyListener, WindowListener
{
	private static TextField input = null;
	
	private InputDialog(Frame parent, String title)
	{
		super(parent, title, true);
		
		setFont(JCards.baseFont);
	}
	
	private InputDialog(Dialog parent, String title)
	{
		super(parent, title, true);
		
		setFont(JCards.baseFont);
	}
	
	/**
	 * Show an input dialog
	 * @param parent owning Window
	 * @param title title of the dialog
	 * @param question question to ask
	 * @param defaultVal a default value to prepopulate
	 */
	public static String showInputDialog(Window parent, String title, String question, String defaultVal)
	{
		ResourceBundle buttons = JCards.getButtonsBundle();
		
		InputDialog inputDialog;
		if (parent instanceof Frame) {
			inputDialog = new InputDialog((Frame)parent, title);
		} else {
			inputDialog = new InputDialog((Dialog)parent, title);
		}
		
		inputDialog.addWindowListener(inputDialog);
		
		inputDialog.setLayout(new BorderLayout());

		Panel inputPanel = new Panel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		
		Label questionLabel = new Label(question);
		gbc.gridwidth = 1;
		gbc.insets = new Insets(5, 5, 5, 10);
		inputPanel.add(questionLabel, gbc);
		
		input = new TextField(10);
		input.addKeyListener(inputDialog);
		if (defaultVal != null) input.setText(defaultVal);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1;
		gbc.insets = new Insets(5, 0, 5, 5);
		inputPanel.add(input, gbc);
		
		Panel buttonPanel = new Panel(new GridLayout(1, 2, 5, 2));
		
		JCButton okButton = new JCButton(buttons.getString("ok"), JCards.getImage(inputDialog, UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		okButton.setActionCommand("ok");
		okButton.addActionListener(inputDialog);
		buttonPanel.add(okButton);
		
		JCButton cancelButton = new JCButton(buttons.getString("cancel"), JCards.getImage(inputDialog, UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(inputDialog);
		buttonPanel.add(cancelButton);
		
		inputDialog.add(inputPanel, BorderLayout.CENTER);
		inputDialog.add(buttonPanel, BorderLayout.SOUTH);
		
		inputDialog.pack();
		JCards.centreWindow(parent, inputDialog);
		inputDialog.show();
		inputDialog.dispose();
		
		return input.getText();
	}
	
	/**
	 * Show an input dialog
	 * @param parent owning Window
	 * @param title title of the dialog
	 * @param question question to ask
	 */
	public static String showInputDialog(Window parent, String title, String question)
	{
		return showInputDialog(parent, title, question, null);
	}
	
	/** Set focus to the input field */
	public void show()
	{
		super.show();
		input.requestFocus();
	}
	
	/** User clicked one of the buttons */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();
		
		if (ac.equals("cancel")) {
			input.setText(null);
		}
		
		dispose();
	}
	
	/** User clicked close window decoration */
	public void windowClosing(WindowEvent e)
	{
		dispose();
	}

	/** Select any text */
	public void windowOpened(WindowEvent e) {
		if (input.getText().length() > 0) {
			input.select(0, input.getText().length());
		}
	}
	
	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	
	
	/** User pressed enter or escape */
	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			actionPerformed(new JCActionEvent(this, 0, "ok"));
		}
		else if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			actionPerformed(new JCActionEvent(this, 0, "cancel"));
		}
	}

	public void keyPressed(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {}	
}
