package nrs.jcards.dialog;

import java.awt.Dialog;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ResourceBundle;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Password dialog
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class PasswordDialog extends Dialog implements JCActionListener, KeyListener
{
	private JCards jcards = null;
	private TextField passTF1=new TextField(10), passTF2=new TextField(10);
	private boolean verify = true;
	
	/**
	 * Creates a new instance of PasswordDialog
	 * @param jcards app class
	 * @param currentPassword the current password or null if not set
	 * @param verify if true use two lines for verifying the password, otherwise
	 * just enter it once
	 */
	public PasswordDialog(JCards jcards, String currentPassword, boolean verify)
	{
		super(jcards, true);
		
		this.jcards = jcards;
		this.verify = verify;
		
		ResourceBundle buttons = JCards.getButtonsBundle();
		ResourceBundle labels = JCards.getLabelsBundle();
		
		setFont(jcards.baseFont);
		setTitle(labels.getString("enter.password"));
		
		if (verify) setLayout(new GridLayout(3, 2, 2, 2));
		else setLayout(new GridLayout(2, 2, 2, 2));
		
		add(new Label(labels.getString("enter.password")+":"));
		passTF1.setEchoChar('*');
		passTF1.addKeyListener(this);
		if (currentPassword.length() > 0) passTF1.setText(currentPassword);
		add(passTF1);
		
		if (verify)
		{
			add(new Label(labels.getString("reenter.password")+":"));
			passTF2.setEchoChar('*');
			passTF2.addKeyListener(this);
			if (currentPassword != null) passTF2.setText(currentPassword);
			add(passTF2);
		}
		
		JCButton okButton = new JCButton(buttons.getString("ok"), JCards.getImage(this, UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		add(okButton);
		
		JCButton cancelButton = new JCButton(buttons.getString("cancel"), JCards.getImage(this, UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		add(cancelButton);
	}
	
	/**
	 * OK or Cancel is clicked.
	 * Passwords must match if verifying
	 * @param e description of the event
	 */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();
		
		if (ac.equals("ok"))
		{
			String pass1 = passTF1.getText();
			String pass2 = passTF2.getText();
			
			if (verify)
			{
				if (pass1.equals(pass2)) {
					jcards.setPassword(pass1);
				} else {
					InfoDialog.showInfoDialog(jcards, jcards.getMessage("incorrect.password"), InfoDialog.DIALOG_INFO, jcards.getMessage("pass.no.match"));
					return;
				}
			} else	{
				jcards.setPassword(pass1);
			}
		}
		
		dispose();
	}
	
	/** Set focus to first password field */
	public void show()
	{
		super.show();
		passTF1.requestFocus();
	}
	
	/** Detect return or escape */
	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			actionPerformed(new JCActionEvent(this, 0, "ok"));
		}
		else if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			actionPerformed(new JCActionEvent(this, 0, "cancel"));
		}
	}
	
	public void keyTyped(KeyEvent e) {}
	public void keyPressed(KeyEvent e) {}	
}
