package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import jclass.bwt.*;
import nrs.jcards.*;

/*
 * JCards
 * Copyright (C) 2002-2004 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Preferences dialog window
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class PrefDialog extends Dialog implements ItemListener, JCActionListener, WindowListener
{
	private JCards jcards;
	private TextField userDate;
	private Choice fontName, fontSize, fontStyle;
	private TextArea fontExample;
	private Choice dateChoice, sortChoice, secondSortChoice, sortDir;
	private Checkbox todayCB, confirmSaveCB, openLastCB, autoEditCB, wrapCB, hideCB;

	private List allFieldsList;
	private List selectedFieldsList;
	private List allIconsList;
	private List selectedIconsList;

	private UserSettings settings;

	private boolean changes = false;

	/**
	 * Creates a new instance of PrefDialog
	 * @param jcards the parent frame
	 * @param visibleFields if true preferences dialog will default to visible fields tab
	 */
	public PrefDialog(JCards jcards, boolean visibleFields)
	{
		super(jcards, true);

		this.jcards = jcards;
		settings = jcards.getUserSettings();

		setForeground(Color.black);
		setBackground(Color.white);
		setFont(jcards.baseFont);

		ResourceBundle labels = JCards.getLabelsBundle();
		ResourceBundle buttons = JCards.getButtonsBundle();
		setTitle(labels.getString("prefs"));

		setLayout(new BorderLayout());

		// add toolbar
		JCButton ok = new JCButton(buttons.getString("ok"), jcards.getImage(UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		ok.setActionCommand("ok");
		ok.addActionListener(this);

		JCButton cancel = new JCButton(buttons.getString("cancel"), jcards.getImage(UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancel.setActionCommand("cancel");
		cancel.addActionListener(this);

		Panel buttonPanel = new Panel(new FlowLayout(FlowLayout.LEFT, 1, 0));
		buttonPanel.add(ok);
		buttonPanel.add(cancel);

		add(buttonPanel, BorderLayout.NORTH);

		JCTabManager tabMan = new JCTabManager();
		DBDef def = DBDef.getInstance();

		// app settings
		tabMan.addPage(labels.getString("tab.app"), createAppPanel(labels));

		// database settings
		tabMan.addPage(labels.getString("tab.db"), createDBPanel(def, labels));

		// visible fields
		tabMan.addPage(labels.getString("tab.visfields"), createVFPanel(def, labels));

		// icons
		tabMan.addPage(labels.getString("tab.icons"), createIconPanel(labels));

		add(tabMan, BorderLayout.CENTER);

		// when user closes dialog via window decoration
		addWindowListener(this);

		if (visibleFields) {
			// focus on visible fields tab
			tabMan.setCurrentTab(2, true);
		}
	}

	/** Create the application panel */
	private ScrollPane createAppPanel(ResourceBundle labels)
	{
		Label l;
		GridBagLayout gbl = new GridBagLayout();

		// label constraints
		GridBagConstraints gbcLabel = new GridBagConstraints();
		gbcLabel.anchor = GridBagConstraints.WEST;
		gbcLabel.fill = GridBagConstraints.HORIZONTAL;
		gbcLabel.weightx = 0.0;
		gbcLabel.weighty = 0.0;
		gbcLabel.gridwidth = 1;
		gbcLabel.insets = new Insets(2, 2, 2, 5);

		// component constraints
		GridBagConstraints gbcComp = new GridBagConstraints();
		gbcComp.anchor = GridBagConstraints.WEST;
		gbcComp.fill = GridBagConstraints.HORIZONTAL;
		gbcComp.weightx = 1.0;
		gbcComp.weighty = 0.0;
		gbcComp.gridwidth = GridBagConstraints.REMAINDER;
		gbcComp.insets = new Insets(2, 2, 2, 2);

		/* Application settings */
		Panel appPanel = new Panel(gbl);

		// confirm save
		l = new Label(labels.getString("save.confirm")+":");
		appPanel.add(l, gbcLabel);

		confirmSaveCB = new Checkbox();
		confirmSaveCB.setState(jcards.isSaveConfirmation());
		appPanel.add(confirmSaveCB, gbcComp);

		// load last file
		l = new Label(labels.getString("load.last")+":");
		appPanel.add(l, gbcLabel);

		openLastCB = new Checkbox();
		openLastCB.setState(jcards.isOpenLastFile());
		appPanel.add(openLastCB, gbcComp);

		// auto edit cards
		l = new Label(labels.getString("auto.edit")+":");
		appPanel.add(l, gbcLabel);

		autoEditCB = new Checkbox();
		autoEditCB.setState(jcards.isAutoEdit());
		appPanel.add(autoEditCB, gbcComp);

		// hide fields on view
		l = new Label(labels.getString("word.wrap")+":");
		appPanel.add(l, gbcLabel);

		wrapCB = new Checkbox();
		wrapCB.setState(jcards.isWordWrap());
		appPanel.add(wrapCB, gbcComp);

		// font
		l = new Label(labels.getString("font"));
		Font f = new Font(jcards.baseFont.getName(), Font.BOLD, jcards.baseFont.getSize()+2);
		l.setFont(f);
		gbcLabel.gridwidth = GridBagConstraints.REMAINDER;
		appPanel.add(l, gbcLabel);

		fontName = new Choice();
		fontName.setName("FONT");
		String[] fontNames = Toolkit.getDefaultToolkit().getFontList();
		for (int i=0; i<fontNames.length; i++)
		{
			fontName.add(fontNames[i]);
		}
		fontName.select(f.getName());
		fontName.addItemListener(this);
		gbcLabel.insets = new Insets(2, 2, 0, 2);
		gbcLabel.gridx = 0;
		gbcLabel.gridwidth = 1;
		appPanel.add(fontName, gbcLabel);

		fontSize = new Choice();
		fontSize.setName("FONT");
		for (int i=8; i<=20; i++) {
			fontSize.add(Integer.toString(i));
		}
		fontSize.select(Integer.toString(jcards.baseFont.getSize()));
		fontSize.addItemListener(this);
		appPanel.add(fontSize, gbcLabel);

		fontStyle = new Choice();
		fontStyle.setName("FONT");
		fontStyle.add(labels.getString("font.plain"));
		fontStyle.add(labels.getString("font.bold"));
		fontStyle.add(labels.getString("font.italic"));
		fontStyle.add(labels.getString("font.bold.italic"));
		fontStyle.select(jcards.baseFont.getStyle());
		fontStyle.addItemListener(this);
		gbcLabel.anchor = GridBagConstraints.NORTHWEST;
		gbcLabel.weighty = 1;
		appPanel.add(fontStyle, gbcLabel);

		fontExample = new TextArea("abcdefghijklmnopqrstuvwxyz\nABCDEFGHIJKLMNOPQRSTUVWXYZ\n0123456789", 3, 16);
		fontExample.setEditable(false);
		fontExample.setBackground(Color.white);
		fontExample.setFont(jcards.baseFont);
		gbcComp.gridwidth = GridBagConstraints.REMAINDER;
		gbcComp.gridheight = 3;
		gbcComp.gridx = 1;
		gbcComp.fill = GridBagConstraints.BOTH;
		gbcComp.weightx = 1;
		gbcComp.weighty = 1;
		appPanel.add(fontExample, gbcComp);

		ScrollPane scroll = new ScrollPane();
		scroll.add(appPanel);

		return scroll;
	}

	/** Create the DB panel */
	private Panel createDBPanel(DBDef def, ResourceBundle labels)
	{
		GridBagLayout gbl = new GridBagLayout();
		Panel dbPanel = new Panel(gbl);
		dbPanel.setBackground(Color.green);
		dbPanel.setForeground(Color.black);

		GridBagConstraints gbcLabel = new GridBagConstraints();
		gbcLabel.gridwidth = 1;
		gbcLabel.weightx = 0.0;
		gbcLabel.anchor = GridBagConstraints.WEST;

		GridBagConstraints gbcComp = new GridBagConstraints();
		gbcComp.gridwidth = GridBagConstraints.REMAINDER;
		gbcComp.fill = GridBagConstraints.HORIZONTAL;
		gbcComp.weightx = 1.0;

		// date format
		Label l = new Label(labels.getString("select.date")+":");
		l.setForeground(Color.black);
		gbl.setConstraints(l, gbcLabel);
		dbPanel.add(l);

		dateChoice = new Choice();
		dateChoice.setName("DATE");
		dateChoice.setForeground(Color.black);
		for (int i=0; i<UserSettings.DATE_FORMATS.length; i++)
		{
			dateChoice.add(UserSettings.DATE_FORMATS[i]);
		}
		dateChoice.select(def.getDateFormat());
		dateChoice.addItemListener(this);
		gbl.setConstraints(dateChoice, gbcComp);
		dbPanel.add(dateChoice);

		// user date format
		l = new Label(labels.getString("user.format")+":");
		l.setForeground(Color.black);
		gbl.setConstraints(l, gbcLabel);
		dbPanel.add(l);

		userDate = new TextField(10);
		userDate.setText(def.getDateFormat());
		userDate.setForeground(Color.black);
		userDate.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				changes = true;
			}
 		});
		gbl.setConstraints(userDate, gbcComp);
		dbPanel.add(userDate);

		// prepopulate with today
		l = new Label(labels.getString("use.today")+":");
		l.setForeground(Color.black);
		gbl.setConstraints(l, gbcLabel);
		dbPanel.add(l);

		todayCB = new Checkbox();
		todayCB.setName("TODAY");
		todayCB.setState(def.isDatePrepopulated());
		todayCB.setForeground(Color.black);
		todayCB.addItemListener(this);
		gbl.setConstraints(todayCB, gbcComp);
		dbPanel.add(todayCB);

		// show field labels
		l = new Label(labels.getString("show.labels")+":");
		l.setForeground(Color.black);
		gbl.setConstraints(l, gbcLabel);
		dbPanel.add(l);

		hideCB = new Checkbox();
		hideCB.setName("LABELS");
		hideCB.setState(def.displayFieldNames());
		hideCB.setForeground(Color.black);
		hideCB.addItemListener(this);
		gbl.setConstraints(hideCB, gbcComp);
		dbPanel.add(hideCB);

		// sort field
		l = new Label(labels.getString("sort.on")+":");
		l.setForeground(Color.black);
		gbl.setConstraints(l, gbcLabel);
		dbPanel.add(l);

		sortChoice = new Choice();
		sortChoice.setName("SORTON");
		sortChoice.setForeground(Color.black);

		secondSortChoice = new Choice();
		secondSortChoice.setName("SORTON");
		secondSortChoice.setForeground(Color.black);
		secondSortChoice.add("");

		FieldDef fd;
		Vector fieldDefs = def.getFieldDefs();
		Enumeration enm = fieldDefs.elements();
		while (enm.hasMoreElements()) {
			fd = (FieldDef)enm.nextElement();
			sortChoice.add(fd.getName());
			secondSortChoice.add(fd.getName());
		}
		String sortField = def.getSortField();
		if (sortField != null) sortChoice.select(sortField);
		sortField = def.getSecondSortField();
		if (sortField != null) secondSortChoice.select(sortField);
		sortChoice.addItemListener(this);
		secondSortChoice.addItemListener(this);

		dbPanel.add(sortChoice, gbcComp);

		// second sort field
		l = new Label(labels.getString("sort.on.2")+":");
		l.setForeground(Color.black);
		dbPanel.add(l, gbcLabel);

		dbPanel.add(secondSortChoice, gbcComp);

		// sort dir
		l = new Label(labels.getString("sort.dir")+":");
		l.setForeground(Color.black);
		gbcLabel.weighty = 1.0;
		gbcLabel.anchor = GridBagConstraints.NORTHWEST;
		gbl.setConstraints(l, gbcLabel);
		dbPanel.add(l);

		sortDir = new Choice();
		sortDir.setName("SORTDIR");
		sortDir.add(labels.getString("asc"));
		sortDir.add(labels.getString("desc"));
		sortDir.select(def.getSortDirection());
		sortDir.setForeground(Color.black);
		sortDir.addItemListener(this);
		gbcComp.weighty = 1.0;
		gbcComp.anchor = GridBagConstraints.NORTHWEST;
		gbl.setConstraints(sortDir, gbcComp);
		dbPanel.add(sortDir);

//		ScrollPane sp = new ScrollPane();
//		sp.add(dbPanel);

		return dbPanel;
	}

	/** Create the visible fields panel */
	private Panel createVFPanel(DBDef def, ResourceBundle labels)
	{
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		Panel visPanel = new Panel(gbl);
		visPanel.setBackground(Color.red);
		visPanel.setForeground(Color.white);

		Label l = new Label(labels.getString("all"), Label.CENTER);
		Font f = jcards.baseFont;
		l.setFont(new Font(f.getName(), Font.BOLD, f.getSize()+2));
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 1.0;
		gbc.insets = new Insets(2, 2, 2, 2);
		gbc.anchor = GridBagConstraints.CENTER;
		visPanel.add(l, gbc);

		// spacer
		l = new Label();
		visPanel.add(l, gbc);

		l = new Label(labels.getString("selected"), Label.CENTER);
		l.setFont(new Font(f.getName(), Font.BOLD, f.getSize()+2));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		visPanel.add(l, gbc);

		allFieldsList = new List(5);
		allFieldsList.setForeground(Color.black);
		gbc.gridwidth = 1;
		gbc.gridheight = 5;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.CENTER;
		visPanel.add(allFieldsList, gbc);

		Panel buttons = createIconButtonPanel("field");
		gbc.gridheight = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		gbc.weighty = 1.0;
		visPanel.add(buttons, gbc);

		selectedFieldsList = new List(5);
		selectedFieldsList.setForeground(Color.black);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.gridheight = 5;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		visPanel.add(selectedFieldsList, gbc);

		// reset the fields in each
		resetFields();

		return visPanel;
	}

	/** Create the icons panel */
	private Panel createIconPanel(ResourceBundle labels)
	{
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		Panel iconPanel = new Panel(gbl);
		iconPanel.setBackground(Color.blue);
		iconPanel.setForeground(Color.white);

		Label l = new Label(labels.getString("all"), Label.CENTER);
		Font f = jcards.baseFont;
		l.setFont(new Font(f.getName(), Font.BOLD, f.getSize()+2));
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 1.0;
		gbc.insets = new Insets(2, 2, 2, 2);
		gbc.anchor = GridBagConstraints.CENTER;
		gbl.setConstraints(l, gbc);
		iconPanel.add(l);

		// spacer
		l = new Label();
		gbl.setConstraints(l, gbc);
		iconPanel.add(l);

		l = new Label(labels.getString("selected"), Label.CENTER);
		l.setFont(new Font(f.getName(), Font.BOLD, f.getSize()+2));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(l, gbc);
		iconPanel.add(l);

		allIconsList = new List(5);
		allIconsList.setForeground(Color.black);
		gbc.gridwidth = 1;
		gbc.gridheight = 5;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.CENTER;
		iconPanel.add(allIconsList, gbc);

		Panel buttons = createIconButtonPanel("icon");
		gbc.gridheight = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		gbc.weighty = 1.0;
		iconPanel.add(buttons, gbc);

		selectedIconsList = new List(5);
		selectedIconsList.setForeground(Color.black);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.gridheight = 5;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		iconPanel.add(selectedIconsList, gbc);

		// set the icons in each
		resetIcons();

		return iconPanel;
	}

	/** Create the icon buttons panel */
	private Panel createIconButtonPanel(String actionPrefix)
	{
		Panel p = new Panel(new GridLayout(5, 1, 5, 5));

		JCButton add = new JCButton(null, jcards.getImage(UserSettings.ICON_NEXT), BWTEnum.STRING_RIGHT);
		add.setActionCommand(actionPrefix+"_add");
		add.addActionListener(this);
		p.add(add);

		JCButton remove = new JCButton(null, jcards.getImage(UserSettings.ICON_PREVIOUS), BWTEnum.STRING_RIGHT);
		remove.setActionCommand(actionPrefix+"_remove");
		remove.addActionListener(this);
		p.add(remove);

		JCButton reset = new JCButton(null, jcards.getImage(UserSettings.ICON_RESET), BWTEnum.STRING_RIGHT);
		reset.setActionCommand(actionPrefix+"s_reset");
		reset.addActionListener(this);
		p.add(reset);

		JCButton up = new JCButton(null, jcards.getImage(UserSettings.ICON_UP), BWTEnum.STRING_RIGHT);
		up.setActionCommand(actionPrefix+"_up");
		up.addActionListener(this);
		p.add(up);

		JCButton down = new JCButton(null, jcards.getImage(UserSettings.ICON_DOWN), BWTEnum.STRING_RIGHT);
		down.setActionCommand(actionPrefix+"_down");
		down.addActionListener(this);
		p.add(down);

		return p;
	}

	/** Called when one of the buttons is clicked */
	public void actionPerformed(JCActionEvent actionEvent)
	{
		String cmd = actionEvent.getActionCommand();

		if (cmd.equals("ok"))
		{
			// set app prefs
			jcards.setSaveConfirmation(confirmSaveCB.getState());
			jcards.setOpenLastFile(openLastCB.getState());
			jcards.setAutoEdit(autoEditCB.getState());
			jcards.setWordWrap(wrapCB.getState());

			if (changes)
			{
				// set db prefs
				DBDef def = DBDef.getInstance();
				// check date format entered
				if (!def.setDateFormat(userDate.getText())) {
					InfoDialog.showInfoDialog(jcards, jcards.getMessage("incorrect.date"), InfoDialog.DIALOG_INFO, jcards.getMessage("invalid.date.format"));
					return;
				}
				def.setDatePrepopulated(todayCB.getState());
				def.setVisibleFields(selectedFieldsList.getItems());
				String ssField = secondSortChoice.getSelectedItem();
				if (ssField.length() == 0) ssField = null;
				def.setSortCriteria(sortChoice.getSelectedItem(), ssField, sortDir.getSelectedIndex());
				def.setDisplayFieldNames(hideCB.getState());
			}

			// set font
			jcards.baseFont = getUserFont();

			// set user icons
			String[] icons = selectedIconsList.getItems();
			settings.setUserIconNames(icons);
			jcards.resetToolbar();

			close();

		} else if (cmd.equals("cancel")) {
			close();
		}
		else if (cmd.startsWith("icon"))
		{
			if (cmd.equals("icon_add")) {
				addIcon(selectedIconsList, allIconsList);
			} else if (cmd.equals("icon_remove")) {
				removeIcon(selectedIconsList, allIconsList);
			} else if (cmd.equals("icons_reset")) {
				resetIcons();
			} else if (cmd.equals("icon_up")) {
				iconUp(selectedIconsList);
			} else if (cmd.equals("icon_down")) {
				iconDown(selectedIconsList);
			}
		}
		else if (cmd.startsWith("field"))
		{
			if (cmd.equals("field_add")) {
				addIcon(selectedFieldsList, allFieldsList);
			} else if (cmd.equals("field_remove")) {
				removeIcon(selectedFieldsList, allFieldsList);
			} else if (cmd.equals("fields_reset")) {
				resetFields();
			} else if (cmd.equals("field_up")) {
				iconUp(selectedFieldsList);
			} else if (cmd.equals("field_down")) {
				iconDown(selectedFieldsList);
			}
			changes = true;
		}
	}

	/** Move an icon/field up */
	private void iconUp(List selectedList)
	{
		int index = selectedList.getSelectedIndex();
		if (index < 1) return;

		String iconName = selectedList.getSelectedItem();
		selectedList.remove(index);
		selectedList.add(iconName, index-1);
 		selectedList.select(index-1);
	}

	/** Move an icon/field down */
	private void iconDown(List selectedList)
	{
		int index = selectedList.getSelectedIndex();
		if (index == -1 || index > selectedList.getItemCount()) return;

		String iconName = selectedList.getSelectedItem();
		selectedList.remove(index);
		selectedList.add(iconName, index+1);
 		selectedList.select(index+1);
	}

	/** Add an icon/field to display */
	private void addIcon(List selectedList, List allList)
	{
		int index = allList.getSelectedIndex();
		if (index == -1) return;

		String iconName = allList.getSelectedItem();
		allList.remove(iconName);
		selectedList.add(iconName);

		if (index < allList.getItemCount()) {
			allList.select(index);
		}
	}

	/** Remove an icon/field from display */
	private void removeIcon(List selectedList, List allList)
	{
		int index = selectedList.getSelectedIndex();
		if (index == -1) return;

		String iconName = selectedList.getSelectedItem();
		selectedList.remove(iconName);
		allList.add(iconName);

		if (index < selectedList.getItemCount()) {
			selectedList.select(index);
		}
	}

	/** Reset the field lists */
	private void resetFields()
	{
		Vector fieldNames = DBDef.getInstance().getFieldNames();
		Vector visFields = DBDef.getInstance().getVisibleFields();
		String fieldName;

		selectedFieldsList.removeAll();
		allFieldsList.removeAll();

		Enumeration en = fieldNames.elements();
		while (en.hasMoreElements()) {
			fieldName = (String)en.nextElement();
			if (visFields.contains(fieldName)) {
				selectedFieldsList.add(fieldName);
			} else {
				allFieldsList.add(fieldName);
			}
		}
	}

	/** Reset the icons in each list */
	private void resetIcons()
	{
		String key, name;
		I18NIcon icon;
		Hashtable allIcons = settings.getAllIcons(false);
		Vector allIconsNames = new Vector(allIcons.size());
		Vector userIcons = settings.getUserIcons();
		Vector userIconsNames = new Vector(userIcons.size());
		Enumeration enm;

		// reset user settings
		settings.resetIcons();

		// clear lists
		allIconsList.removeAll();
		selectedIconsList.removeAll();

		// copy all icons to a Vector of names
		enm = allIcons.keys();
		while (enm.hasMoreElements())
		{
			key = (String)enm.nextElement();
			icon = (I18NIcon)allIcons.get(key);
			allIconsNames.addElement(icon.getName());
		}

		// copy user icons to a Vector of names
		enm = userIcons.elements();
		while (enm.hasMoreElements())
		{
			icon = (I18NIcon)enm.nextElement();
			userIconsNames.addElement(icon.getName());
		}

		// split icons into the two lists
		enm = userIconsNames.elements();
		while (enm.hasMoreElements())
		{
			name = (String)enm.nextElement();
			selectedIconsList.add(name);
			allIconsNames.removeElement(name);
		}

		enm = allIconsNames.elements();
		while (enm.hasMoreElements())
		{
			allIconsList.add((String)enm.nextElement());
		}
	}

	/** Close the dialog */
	private void close()
	{
		setVisible(false);
		dispose();
	}

	/**
	 * Get the maximum size of the dialog.
	 * @return max size of dialog = max size of app
	 */
	public Dimension getMaximumSize()
	{
		return jcards.getMaximumSize();
	}

	/**
	 * Get the preferred size of the dialog.
	 * @return max size of dialog = max size of app
	 */
	public Dimension getPreferredSize()
	{
		if (jcards.settings.isPda()) {
			return jcards.getPreferredSize();
		} else {
			return new Dimension(400, 400);
		}
	}

	/** Create a Font based on the choices */
	private Font getUserFont()
	{
		int fs = Font.PLAIN;
		int fsname = fontStyle.getSelectedIndex();
		switch (fsname)
		{
			case 0: fs = Font.PLAIN;
				break;

			case 1: fs = Font.BOLD;
				break;

			case 2: fs = Font.ITALIC;
				break;

			case 3: fs = Font.BOLD | Font.ITALIC;
				break;
		}
		Font f = new Font(fontName.getSelectedItem(), fs,
			Integer.parseInt(fontSize.getSelectedItem()));

		return f;
	}

	/** User selected a date or font or other change */
	public void itemStateChanged(ItemEvent e)
	{
		Component c = (Component)e.getSource();
		String name = c.getName();
		if (name.equals("DATE"))
		{
			String dateF = dateChoice.getSelectedItem();
			userDate.setText(dateF);
			changes = true;
		}
		else if (name.equals("FONT"))
		{
			fontExample.setFont(getUserFont());
			fontExample.setCaretPosition(0);
		}
		else
		{
			changes = true;
		}
	}

	/** Dialog is displayed */
	public void show()
	{
		super.show();
		if (isShowing()) {
			fontExample.setCaretPosition(0);
		}
	}

	/** User clicked the close window decoration */
	public void windowClosing(WindowEvent e) {
		close();
	}

	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
}
