package nrs.jcards.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jclass.bwt.BWTEnum;
import jclass.bwt.JCActionEvent;
import jclass.bwt.JCActionListener;
import jclass.bwt.JCButton;

import nrs.jcards.JCards;
import nrs.jcards.UserSettings;

/*
 * This file is part of JCards.
 * JCards is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JCards is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JCards; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Select a field to collect statistics on
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class StatisticsDialog extends Dialog implements JCActionListener, WindowListener
{
	private JCards jcards = null;
	private Choice fieldChoice;

	/** Creates a new instance of StatisticsDialog */
	public StatisticsDialog(JCards jcards, Vector fieldNames)
	{
		super(jcards, true);

		this.jcards = jcards;

		ResourceBundle buttons = JCards.getButtonsBundle();
		ResourceBundle labels = JCards.getLabelsBundle();

		setFont(JCards.baseFont);
		setTitle(labels.getString("select.field"));

		setLayout(new BorderLayout(2, 2));
		addWindowListener(this);

		// inputs
		Panel inputPanel = new Panel();
		add(inputPanel, BorderLayout.CENTER);

		inputPanel.add(new Label(labels.getString("field")+":"));

		fieldChoice = new Choice();
		Enumeration enm = fieldNames.elements();
		while (enm.hasMoreElements())
		{
			fieldChoice.add((String)enm.nextElement());
		}
		inputPanel.add(fieldChoice);

		// buttons
		Panel buttonPanel = new Panel(new GridLayout(1, 2, 2, 2));
		add(buttonPanel, BorderLayout.SOUTH);

		JCButton cancelButton = new JCButton(buttons.getString("cancel"), JCards.getImage(this, UserSettings.ICON_CANCEL), BWTEnum.STRING_RIGHT);
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);

		JCButton okButton = new JCButton(buttons.getString("ok"), JCards.getImage(this, UserSettings.ICON_OK), BWTEnum.STRING_RIGHT);
		okButton.setActionCommand("OK");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
	}

	/** Invoked when an action occurs. */
	public void actionPerformed(JCActionEvent e)
	{
		String ac = e.getActionCommand();

		if (ac.equals("OK")) {
			jcards.showStatistics(fieldChoice.getSelectedItem());
			windowClosing(null);
		} else if (ac.equals("cancel")) {
			windowClosing(null);
		}
	}

	/** Invoked when the user attempts to close the window
	 * from the window's system menu.  If the program does not
	 * explicitly hide or dispose the window while processing
	 * this event, the window close operation will be cancelled.
	 */
	public void windowClosing(WindowEvent e)
	{
		dispose();
	}

	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
}
