package nrs.jcards.io;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import nanoxml.*;

import nrs.jcards.*;

/*
 * JCards
 * Copyright (C) 2002-2004 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Reads and writes XML for DBDef
 *
 * @author Copyright &#169; 2002-2004 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DBDefXML
{
	/**
	 * Get the data as XML
	 * @param def DBDef
	 * @return XMLElement
	 */
	public static XMLElement getXML(DBDef def)
	{
		FieldDef fd;
		String catName, catVal;
		XMLElement fieldEl, catEl, catItemEl, countEl;
		Vector catItems;
		Counter c;
		Enumeration enm, cenum;
		
		XMLElement dbdEl = new XMLElement();
		dbdEl.setTagName(Elements.DBDEF);
		
		// save settings
		XMLElement setEl = new XMLElement();
		setEl.setTagName(Elements.SETTINGS);
		
		XMLElement dateEl = new XMLElement();
		dateEl.setTagName(Elements.DATE_FORMAT);
		dateEl.setContent(def.getDateFormat());
		setEl.addChild(dateEl);
		
		XMLElement prepEl = new XMLElement();
		prepEl.setTagName(Elements.PREPOPULATE_DATE);
		Boolean b = new Boolean(def.isDatePrepopulated());
		prepEl.setContent(b.toString());
		setEl.addChild(prepEl);
		
		XMLElement dfnEl = new XMLElement();
		dfnEl.setTagName(Elements.DISPLAY_FIELD_NAMES);
		b = new Boolean(def.displayFieldNames());
		dfnEl.setContent(b.toString());
		setEl.addChild(dfnEl);
		
		// sort fields
		if (def.getSortField() != null)
		{
			XMLElement sortEl = new XMLElement();
			sortEl.setTagName(Elements.SORT_FIELD);
			sortEl.setContent(def.getSortField());
			sortEl.addProperty(Elements.SORT_DIR, def.getSortDirection());
			setEl.addChild(sortEl);
		}
		
		if (def.getSecondSortField() != null)
		{
			XMLElement secondSortEl = new XMLElement();
			secondSortEl.setTagName(Elements.SECOND_SORT_FIELD);
			secondSortEl.setContent(def.getSecondSortField());
			setEl.addChild(secondSortEl);
		}
		
		// visible fields
		if (def.getVisibleFields().size() > 0)
		{
			XMLElement visEl = new XMLElement();
			visEl.setTagName(Elements.VIS_FIELDS);
			XMLElement vfEl;
			enm = def.getVisibleFields().elements();
			while (enm.hasMoreElements())
			{
				vfEl = new XMLElement();
				vfEl.setTagName(Elements.VIS_FIELD);
				vfEl.setContent((String)enm.nextElement());
				visEl.addChild(vfEl);
			}
			setEl.addChild(visEl);
		}
		dbdEl.addChild(setEl);
		
		// save Categories
		XMLElement catsEl = new XMLElement();
		catsEl.setTagName(Elements.CATEGORIES);
		enm = def.getCategories().keys();
		while (enm.hasMoreElements())
		{
			catName = (String)enm.nextElement();
			catEl = new XMLElement();
			catEl.setTagName(Elements.CATEGORY);
			catEl.addProperty(Elements.NAME, catName);
			
			// add category items
			catItems = def.getCategory(catName);
			cenum = catItems.elements();
			while (cenum.hasMoreElements())
			{
				catVal = (String)cenum.nextElement();
				catItemEl = new XMLElement();
				catItemEl.setTagName(Elements.CAT_ITEM);
				catItemEl.setContent(catVal);
				catEl.addChild(catItemEl);
			}
			catsEl.addChild(catEl);
		}
		dbdEl.addChild(catsEl);
		
		// save Counters
		XMLElement countersEl = new XMLElement();
		countersEl.setTagName(Elements.COUNTERS);
		Hashtable counters = def.getCounters();
		enm = counters.keys();
		while (enm.hasMoreElements())
		{
			countEl = new XMLElement();
			countEl.setTagName(Elements.COUNTER);
			c = (Counter)counters.get(enm.nextElement());
			countEl.addProperty(Elements.NAME, c.getName());
			countEl.addProperty(Elements.VALUE, c.getValue());
			countEl.addProperty(Elements.INCREMENT, c.getIncrement());
			countersEl.addChild(countEl);
		}
		dbdEl.addChild(countersEl);
		
		// save Fields
		XMLElement fieldsEl = new XMLElement();
		fieldsEl.setTagName(Elements.FIELDS);
		enm = def.getFieldDefs().elements();
		while (enm.hasMoreElements())
		{
			fd = (FieldDef)enm.nextElement();
			fieldEl = fd.getXML();
			fieldsEl.addChild(fieldEl);
		}
		dbdEl.addChild(fieldsEl);
		
		return dbdEl;
	}
	
	/**
	 * Read data retrieved from a file
	 * @param def the DBDef to setup
	 * @param xmlImport the data to import
	 * @exception InvalidFieldTypeException unknown field type detected
	 * @exception InvalidFieldException field definition is incorrect
	 */
	public static void setXML(DBDef def, XMLElement xmlImport) throws InvalidFieldTypeException, InvalidFieldException
	{
		XMLElement child;
		String tagName;
		
		Vector children = xmlImport.getChildren();
		Enumeration enm = children.elements();
		while (enm.hasMoreElements())
		{
			child = (XMLElement)enm.nextElement();
			tagName = child.getTagName();
			
			if (tagName.equalsIgnoreCase(Elements.SETTINGS))
			{
				XMLElement setXML;
				String setting;
				Vector settings = child.getChildren();
				Enumeration setEnum = settings.elements();
				while (setEnum.hasMoreElements())
				{
					setXML = (XMLElement)setEnum.nextElement();
					setting = setXML.getTagName();
					if (setting.equalsIgnoreCase(Elements.DATE_FORMAT)) { // date format
						def.setDateFormat(setXML.getContents());
					} else if (setting.equalsIgnoreCase(Elements.PREPOPULATE_DATE)) { // prepopulate date
						Boolean prepDate = new Boolean(setXML.getContents());
						def.setDatePrepopulated(prepDate.booleanValue());
					} else if (setting.equalsIgnoreCase(Elements.DISPLAY_FIELD_NAMES)) { // display field names
						Boolean dfn = new Boolean(setXML.getContents());
						def.setDisplayFieldNames(dfn.booleanValue());
					} else if (setting.equalsIgnoreCase(Elements.SORT_FIELD)) { // sort field
						def.setSortField(setXML.getContents(), setXML.getProperty(Elements.SORT_DIR, 0));
					} else if (setting.equalsIgnoreCase(Elements.SECOND_SORT_FIELD)) { // second sort field
						def.setSecondSortField(setXML.getContents());
					} else if (setting.equalsIgnoreCase(Elements.VIS_FIELDS)) { // visible fields
						XMLElement visFieldXML;
						Vector visFields = setXML.getChildren();
						String[] visibleFields = new String[visFields.size()];
						int i = 0;
						Enumeration visEnum = visFields.elements();
						while (visEnum.hasMoreElements())
						{
							visFieldXML = (XMLElement)visEnum.nextElement();
							visibleFields[i++] = visFieldXML.getContents();
						}
						def.setVisibleFields(visibleFields);
					}
				}
			}
			else if (tagName.equalsIgnoreCase(Elements.CATEGORIES)) // categories
			{
				Vector cats = child.getChildren();
				if (cats.size()  > 0)
				{
					Vector catItems, items;
					Enumeration catsEnum = cats.elements();
					Enumeration catEnum;
					XMLElement catXML, catItemXML;
					Hashtable categories = new Hashtable(cats.size());
					while (catsEnum.hasMoreElements())
					{
						catXML = (XMLElement)catsEnum.nextElement();
						catItems = catXML.getChildren();
						items = new Vector(catXML.countChildren());
						catEnum = catItems.elements();
						while (catEnum.hasMoreElements())
						{
							catItemXML = (XMLElement)catEnum.nextElement();
							items.addElement(catItemXML.getContents());
						}
						categories.put(catXML.getProperty(Elements.NAME), items);
					}
					def.setCategories(categories);
				}
			}
			else if (tagName.equalsIgnoreCase(Elements.COUNTERS)) // counters
			{
				Vector counts = child.getChildren();
				if (counts.size() > 0)
				{
					Enumeration countEnum = counts.elements();
					Hashtable counters = new Hashtable(counts.size());
					Counter c;
					XMLElement countXML;
					while (countEnum.hasMoreElements())
					{
						countXML = (XMLElement)countEnum.nextElement();
						if (countXML.getTagName().equalsIgnoreCase(Elements.COUNTER))
						{
							try {
								c = new Counter(countXML.getProperty(Elements.NAME),
															countXML.getProperty(Elements.VALUE),
															countXML.getProperty(Elements.INCREMENT));
								counters.put(countXML.getProperty(Elements.NAME), c);
							} catch (NumberFormatException nfex) {
								JCards.log("Incorrect Counter: "+nfex.getMessage(), 0);
							}
						}
					}
					def.setCounters(counters);
				}
			}
			else if (tagName.equalsIgnoreCase(Elements.FIELDS)) // fields
			{
				Vector fields = child.getChildren();
				if (fields.size() > 0)
				{
					XMLElement fieldXML;
					FieldDef fieldDef;
					Vector fieldDefs = new Vector(fields.size());
					Enumeration fieldEnum = fields.elements();
					while (fieldEnum.hasMoreElements())
					{
						fieldXML = (XMLElement)fieldEnum.nextElement();
						fieldDef = new FieldDef(fieldXML);
						fieldDefs.addElement(fieldDef);
					}
					def.setFieldDefs(fieldDefs);
				}
			}
		}
		
		if (def.getFieldDefs().size() > 0)
		{
			String firstFieldName = def.getFieldDef(0).getName();
			
			// add one field if not specified
			if (def.getVisibleFields().size() == 0)
			{
				String[] visibleFields = { firstFieldName };
				def.setVisibleFields(visibleFields);
			}
			
			// select the first field to be sorted on if not specified
			if (def.getSortField() == null)
			{
				def.setSortField(firstFieldName, Cards.SORT_ASCENDING);
			}
		}
	}
}
